
import UIKit
import AsyncDisplayKit

class AnimalTableNodeController: ASViewController<ASDisplayNode> {
  var animals: [RainforestCardInfo]!
  let tableNode: ASTableNode!
  
  init(animals: [RainforestCardInfo]) {
    let tableNode = ASTableNode(style: .plain)
    self.animals = animals
    self.tableNode = tableNode
    
    super.init(node: tableNode)
    
    self.tableNode.delegate = self
    self.tableNode.dataSource = self
  }
  
  required init?(coder aDecoder: NSCoder) {
    fatalError("init(coder:) has not been implemented")
  }
}

// MARK: - ASTableDelegate

extension AnimalTableNodeController: ASTableDelegate {
  func tableView(_ tableView: ASTableView, willBeginBatchFetchWith context: ASBatchContext) {
    nextPageWithCompletion { (results) in
      self.insertNewRows(results)
      context.completeBatchFetching(true)
    }
  }
  
  func shouldBatchFetch(for tableView: ASTableView) -> Bool {
    return true
  }
  
  func tableView(_ tableView: ASTableView, constrainedSizeForRowAt indexPath: IndexPath) -> ASSizeRange {
    return ASSizeRangeMake(CGSize(width: UIScreen.main.bounds.size.width, height: (UIScreen.main.bounds.size.height/3) * 2), CGSize(width: UIScreen.main.bounds.size.width, height: .greatestFiniteMagnitude))
  }
}

// MARK: - ASTableDataSource

extension AnimalTableNodeController: ASTableDataSource {
  func numberOfSections(in tableView: UITableView) -> Int {
    return 1
  }
  
  func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
    return animals.count
  }
  
  func tableView(_ tableView: ASTableView, nodeBlockForRowAt indexPath: IndexPath) -> ASCellNodeBlock {
    let animal = animals[(indexPath as NSIndexPath).row]
    
    return {
      let node = CardCellNode(animalInfo: animal)
      return node
    }
  }
}

// MARK: - Helpers

extension AnimalTableNodeController {
  func nextPageWithCompletion(_ block: @escaping (_ results: [RainforestCardInfo]) -> ()) {
    let moreAnimals = Array(self.animals[0 ..< 5])
    
    DispatchQueue.main.async {
      block(moreAnimals)
    }
  }
  
  func insertNewRows(_ newAnimals: [RainforestCardInfo]) {
    let section = 0
    var indexPaths = [IndexPath]()
    
    let newTotalNumberOfPhotos = animals.count + newAnimals.count
    
    for row in animals.count ..< newTotalNumberOfPhotos {
      let path = IndexPath(row: row, section: section)
      indexPaths.append(path)
    }
    
    animals.append(contentsOf: newAnimals)
    if let tableView = node.view as? ASTableView {
      tableView.insertRows(at: indexPaths, with: .none)
    }
  }
}
